<?php

namespace Drupal\uber_affiliate\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Component\Utility\Html;
use Drupal\Core\Database\Database;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Component\Utility\Xss;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Access\AccessResult;
use Drupal\uber_affiliate\Form\PayoutForm;
use Drupal\Core\Render\RendererInterface;
use Drupal\uber_affiliate\UberAffiliateTrait;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\HttpFoundation\Response;


/**
 * Affiliate Overview page class.
 */
class UberAffiliate extends  ControllerBase {
  use UberAffiliateTrait;

  /**
   * Admin overview page method.
   */
  public function affiliate_admin_overviewpage() {
    $account = \Drupal::currentUser();
    $payouts_symbol = '';
    $payouts_symbol_pos = '';

    $database = \Drupal::database();
    $query = $database->select('affiliate', 'a');
    $query->innerjoin('users_field_data', 'u');
    $query->condition('u.uid',  0, '<>');
    $query->fields('u', ['uid', 'name', 'status', 'created', 'access']);
    $query->range(0, 50);
    $result = $query->execute();

    $rows = array();

    foreach ($result as $affiliate) {
      $affiliate_uid = $affiliate->uid;

      $affiliate_name = Html::escape($affiliate->name);
      $affiliate_uid_display =  Link::createFromRoute($affiliate_uid, \Drupal::service('path_alias.manager')->getAliasByPath('/user/' . $affiliate_uid));
      $affiliate_uid_display_link = $affiliate_uid_display->toRenderable();
      $affiliate_name_display =  Link::createFromRoute($affiliate_name, \Drupal::service('path_alias.manager')->getAliasByPath('/user/' . $affiliate_uid));
      $affiliate_name_display_link = $affiliate_name_display->toRenderable();
      $affiliate_active = $affiliate->active ? t('Yes') : t('No');
      $affiliate_created = \Drupal::service('date.formatter')->format($affiliate->created);
      $payouts_owed_display = $payouts_symbol . $affiliate->payouts_owed;
      $payouts_paid_display = $payouts_symbol . $affiliate->payouts_paid;
      $payouts_form = '';
      if ($payouts_symbol_pos == 2) {
        $payouts_owed_display = $affiliate->payouts_owed . $payouts_symbol;
        $payouts_paid_display = $affiliate->payouts_paid . $payouts_symbol;
      }
      if (AccessResult::allowedIf($account->hasPermission('administer affiliate settings')) || AccessResult::allowedIf($account->hasPermission('administer affiliate payouts'))) {
        $form = \Drupal::formBuilder()->getForm(PayoutForm::class, $affiliate_uid);
        $renderer = \Drupal::service('renderer');
        $payouts_form .= $this->renderPlain($form, $renderer);
      }
      $rows[] = array(render($affiliate_uid_display_link), render($affiliate_name_display_link), $affiliate_active, $affiliate_created, $affiliate->clicks, $affiliate->referrals, $payouts_owed_display, $payouts_paid_display, $payouts_form);
    }

    $header = [
      'title' => t('ID'),
      'name' => t('Name'),
      'active' => t('Active'),
      'created' => t('Created'),
      'clicks' => t('Clicks'),
      'referrals' => t('Referrals'),
      'payouts_owed' => t('Payouts owed'),
      'payouts_paid' => t('Payouts paid'),
    ];

    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No content has been found.'),
    ];
    return [
      '#type' => '#markup',
      '#markup' => \Drupal::service('renderer')->render($build)
    ];

  }

  /**
   * Displays the referral information of the said user.
   *
   * @param $uid
   * @return Response
   *  Rendered array of the users information.
   */
  public function uber_affiliate_user_page($uid) {
    $currentUser = \Drupal::currentUser();
    $currentUserid = $currentUser->id();
    $acctid = $uid;
    if (($currentUserid == $acctid && $currentUser->hasPermission('view own affiliate info')) || $currentUser->hasPermission('administer affiliate settings')) {
      $name = $currentUser->getAccountName();
      $output = $this->affiliateUserPageOutput($acctid);
      $title = $this->t('Affiliate information for @name', ['@name' => $name]);

      $renderedArray = \Drupal::service('renderer')->render($output);

      return [
        '#markup' => $renderedArray,
        '#title' => $title,
      ];
    }
    else {
      return new Response(t('Access denied'), 403);
    }
  }


  /**
   * Displays the top users.
   * @return mixed
   */
  public function uber_affiliate_top_users_page() {
    $limit = \Drupal::config('affiliate_module.settings')->get('top_users_count_page');
    $time = time();
    $timeInterval = \Drupal::config('affiliate_module.settings')->get('top_users_period_interval');
    $timeLimit = $time - $timeInterval;

    $query = Database::getConnection()->select('affiliate_clicks', 'ac');
    $query->addExpression('COUNT(DISTINCT ac.clickid)', 'clicks');
    $query->join('users_field_data', 'u', 'ac.uid = u.uid');
    $query->join('affiliate', 'aa', 'u.uid = aa.uid');
    $query->fields('u', ['uid', 'name']);
    $query->fields('aa', ['payouts_owed']);
    $query->condition('aa.active', 1);
    $query->condition('ac.click_time', $timeLimit, '>');
    $query->groupBy('u.uid');
    $query->orderBy('clicks', 'DESC');
    $results = $query->execute()->fetchAll();

    $rows = [];
    $rank = 1;
    if ($results && !empty($results)) {
      foreach ($results as $key => $result) {
        $rowClass = ($key % 2) ? 'affiliate-row-even' : 'affiliate-row-odd';
        $userData = unserialize($result->data);
        $homepage = !empty($userData['affiliate_homepage']) ? check_url($userData['affiliate_homepage']) : '';
        $username = Html::escape($result->name);
        $name = !empty($homepage) ? Link::fromTextAndUrl($username, Url::fromUri($homepage))->toString() : Link::fromTextAndUrl($username, Url::fromRoute('entity.user.canonical', ['user' => $result->uid]))->toString();
        $rows[] = [
          ['class' => [$rowClass, 'affiliate-row-rank'], 'data' => $rank++],
          ['class' => [$rowClass, 'affiliate-row-name'], 'data' => $name],
          ['class' => [$rowClass, 'affiliate-row-clicks'], 'data' => $result->clicks],
          ['class' => [$rowClass, 'affiliate-row-payouts-owed'], 'data' => $result->payouts_owed],
        ];
      }
    }
    if (empty($rows)) {
      $rows[] = [['data' => $this->t('No affiliate activity found for this time period.'), 'colspan' => '4']];
    }

    $header = ['Rank', 'User', 'Clicks', 'Payouts'];
    $table = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => ['class' => ['top-affiliates-table']],
    ];

    $build = [
      '#markup' => $renderedArray = \Drupal::service('renderer')->render($table),
   ];

    $timeLimitFormatted = date('Y-m-d', $timeLimit);
    $timeFormatted = date('Y-m-d', $time);
    $title = $this->t('Top affiliates from @from through @to', [
      '@from' => $timeLimitFormatted,
      '@to' => $timeFormatted,
    ]);

    $build['#title'] = $title;
    return $build;
  }
  /**
   * The primary tracking of the affiliate and the main affiliate link.
   * @todo
   *   Have this function checked by security team if possible, or if not, get a second opinion just to be sure.
   */
  public function affiliate_page($aff_id, $dest_path, $tracker_id)
  {
    $allow_all_paths = \Drupal::state()->get('affiliate_module_allow_all_paths', 0);
    $allow_all_node_types = \Drupal::state()->get('affiliate_module_allow_all_node_types', 0);


    $affid = (int)Xss::filter($aff_id);
    $path_given = $dest_path;
    $tracker = (int)Xss::filter($tracker_id);


    // Note to self: ask a security guru about this.
    $path_arg = UrlHelper::stripDangerousProtocols(implode('/', explode('|', rawurldecode($path_given))));

    $verify_node_type = FALSE;
    if (!$allow_all_node_types) {
      $verify_node_type = TRUE;
    }
    if ($allow_all_paths) {
      $verify_node_type = FALSE;
    }

    if (!isset($tracker) || empty($tracker)) {
      $tracker = 0;
    }
    if (!isset($affid) || empty($affid)) {
      $affid = 1;
    }

    // Send invalid clicks to the front page if no path argument is supplied.
    if (!isset($path_arg) || empty($path_arg)) {
      $path = base_path();
    } else {
      $path = $path_arg;
    }

    // Path aliases will fail validation... Convert to normal path before validating.
    $path_normal = \Drupal::service('path_alias.manager')->getPathByAlias($path);


    // This seems like it could be written better...?
    if (affiliate_validate_click($affid, $path_normal, $verify_node_type)) {
      $campaign_id = affiliate_get_campaign_id($path_normal);
      affiliate_add_click($affid, $campaign_id);
    }
    $destination_is_valid = affiliate_valid_destination($path_normal);

    // Empty will simply take us to the front page, in case the path doesn't validate.
    $destination_actual = '';
    if ($destination_is_valid) {
      $destination_actual = \Drupal::service('path_alias.manager')->getPathByAlias($path_normal);
    }

    // Unnecessary check_url()? Better safe than sorry...
    $destination_actual = UrlHelper::stripDangerousProtocols($destination_actual);

    // At minimum, if all validations fail, we'll be taken to the front page. If the path
    // given is valid though, we'll be taken to that path either way, regardless of whether
    // the affiliate was credited with a valid click-thru.
    if (empty($destination_actual)) {
      $language = \Drupal::languageManager()->getLanguage('vi');
      $url = Url::fromRoute('<front>', [], ['language' => $language]);
      return new RedirectResponse($url->toString());

    }
    return new RedirectResponse(Url::fromRoute($destination_actual)->toString());
  }
  /**
   * Renders a form using the renderer service.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   *
   * @return string
   *   The rendered form HTML.
   */
  function renderPlain($form, RendererInterface $renderer) {
    $output = $renderer->renderRoot($form);
    $renderer->renderPlain($form);

    return $output;
  }
}
